﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;

namespace IPv6Multicast {
    public partial class MainForm : Form {

        public MainForm() {
            InitializeComponent();

            this.Socket = new Socket(AddressFamily.InterNetworkV6, SocketType.Dgram, ProtocolType.Udp);
            this.Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
            this.Socket.Bind(new IPEndPoint(IPAddress.IPv6Any, this.MulticastPort));
            this.Socket.SetSocketOption(SocketOptionLevel.IPv6, SocketOptionName.AddMembership, new IPv6MulticastOption(this.MulticastAddress));
        }

        private readonly Socket Socket;
        private readonly IPAddress MulticastAddress = IPAddress.Parse("ff08::1");
        private readonly int MulticastPort = 65535;


        private void MainForm_Load(object sender, EventArgs e) {
            UpdateColor();
            bwReceive.RunWorkerAsync();
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e) {
            this.Socket.SetSocketOption(SocketOptionLevel.IPv6, SocketOptionName.DropMembership, new IPv6MulticastOption(this.MulticastAddress));
        }


        private void chb_Click(object sender, EventArgs e) {
            this.Socket.SendTo(new byte[] { (byte)(chb.Checked ? 'F' : 'T') }, new IPEndPoint(this.MulticastAddress, this.MulticastPort));
        }

        private void chb_CheckedChanged(object sender, EventArgs e) {
            UpdateColor();
        }

        private void UpdateColor() {
            chb.BackColor = chb.Checked ? Color.LightGreen : Color.LightPink;
            chb.Text = chb.Checked ? "ON" : "OFF";
        }


        private void bwReceive_DoWork(object sender, DoWorkEventArgs e) {
            EndPoint remoteEP = (EndPoint)(new IPEndPoint(IPAddress.IPv6Any, 0));
            while (true) {
                byte[] buffer = new Byte[4096];
                this.Socket.ReceiveFrom(buffer, ref remoteEP);
                if (buffer[0] == (byte)'F') {
                    bwReceive.ReportProgress(0);
                } else if (buffer[0] == (byte)'T') {
                    bwReceive.ReportProgress(1);
                }
            }
        }

        private void bwReceive_ProgressChanged(object sender, ProgressChangedEventArgs e) {
            chb.Checked = (e.ProgressPercentage != 0);
        }

    }
}
